/* jcifs smb client library in Java
 * Copyright (C) 2002  "Michael B. Allen" <jcifs at samba dot org>
 *					   "Eric Glass" <jcifs at samba dot org>
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package jcifs.http;

import java.security.Principal;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletRequestWrapper;

import jcifs.smb.NtlmPasswordAuthentication;

class NtlmHttpServletRequest extends HttpServletRequestWrapper {

	Principal principal;

	NtlmHttpServletRequest( HttpServletRequest req, Principal principal ) {
		super( req );
		this.principal = principal;
	}
	public String getRemoteUser() {
		return principal.getName();
	}
	public Principal getUserPrincipal() {
		return principal;
	}
	public String getAuthType() {
		return "NTLM";
	}
    public boolean isUserInRole(String role) {
        if (role == null ||
                !(principal instanceof NtlmPasswordAuthentication)) {
            return super.isUserInRole(role);
        }
        if (super.isUserInRole(role)) return true;
        String[] groups = ((NtlmPasswordAuthentication) principal).getGroups();
        if (groups == null) return false;
        for (int i = groups.length - 1; i >= 0; i--) {
            if (role.equalsIgnoreCase(groups[i])) return true;
        }
        return false;
    }
}

