/* jcifs smb client library in Java
 * Copyright (C) 2002  "Michael B. Allen" <jcifs at samba dot org>
 *					   "Eric Glass" <jcifs at samba dot org>
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package jcifs.rap;

import jcifs.smb.NtlmPasswordAuthentication;

/**
 * Provides simplified interfaces to managing domain groups (NOT machine-local
 * groups).  Note that most of these operations must be performed against the
 * primary domain controller (which will be located automatically).
 * For group creation/deletion etc., the account must have proper domain
 * administrative permissions assigned.
 */
public class GroupUtilities {

    public static String[] getGroups(String domain,
            NtlmPasswordAuthentication auth) {
        try {
            String pdc = ServerUtilities.getPrimaryController(domain, auth);
            if (pdc == null) pdc = domain;
            GroupInfo[] groups = new NetGroupEnum(pdc, auth).getGroups(0);
            if (groups == null) return new String[0];
            int length = groups.length;
            String[] names = new String[length];
            for (int i = 0; i < length; i++) names[i] = groups[i].name;
            return names;
        } catch (Exception ex) {
            return null;
        }
    }

    public static boolean create(String domain, String group,
            NtlmPasswordAuthentication auth) {
        try {
            String pdc = ServerUtilities.getPrimaryController(domain, auth);
            if (pdc == null) pdc = domain;
            GroupInfo info = new GroupInfo();
            info.name = group;
            new NetGroupAdd(pdc, auth).createGroup(info);
            return true;
        } catch (Exception ex) {
            return false;
        }
    }

    public static boolean delete(String domain, String group,
            NtlmPasswordAuthentication auth) {
        try {
            String pdc = ServerUtilities.getPrimaryController(domain, auth);
            if (pdc == null) pdc = domain;
            new NetGroupDel(pdc, auth).deleteGroup(group);
            return true;
        } catch (Exception ex) {
            return false;
        }
    }

    public static String[] getUsers(String domain, String group,
            NtlmPasswordAuthentication auth) {
        try {
            String pdc = ServerUtilities.getPrimaryController(domain, auth);
            if (pdc == null) pdc = domain;
            GroupUsersInfo[] users = new NetGroupGetUsers(pdc, auth).getUsers(
                    group, 0);
            if (users == null) return new String[0];
            int length = users.length;
            String[] names = new String[length];
            for (int i = 0; i < length; i++) names[i] = users[i].name;
            return names;
        } catch (Exception ex) {
            return null;
        }
    }

    public static boolean add(String domain, String group, String user,
            NtlmPasswordAuthentication auth) {
        try {
            String pdc = ServerUtilities.getPrimaryController(domain, auth);
            if (pdc == null) pdc = domain;
            new NetGroupAddUser(pdc, auth).addUser(group, user);
            return true;
        } catch (Exception ex) {
            return false;
        }
    }

    public static boolean remove(String domain, String group, String user,
            NtlmPasswordAuthentication auth) {
        try {
            String pdc = ServerUtilities.getPrimaryController(domain, auth);
            if (pdc == null) pdc = domain;
            new NetGroupDelUser(pdc, auth).removeUser(group, user);
            return true;
        } catch (Exception ex) {
            return false;
        }
    }

}
