/* jcifs smb client library in Java
 * Copyright (C) 2002  "Michael B. Allen" <jcifs at samba dot org>
 *					   "Eric Glass" <jcifs at samba dot org>
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package jcifs.rap;

import java.io.IOException;

import jcifs.smb.NtlmPasswordAuthentication;
import jcifs.smb.Rap;
import jcifs.smb.RapException;
import jcifs.smb.SmbFile;

public class NetGroupAdd {

    private static final int NET_GROUP_ADD = 48;

    private SmbFile target;

    public NetGroupAdd(String target, NtlmPasswordAuthentication auth) {
        try {
            this.target = new SmbFile("smb://" + target + "/", auth);
        } catch (Exception ex) {
            throw new IllegalArgumentException("Invalid target: " + ex);
        }
    }

    public void createGroup(GroupInfo info) throws IOException {
        int result = Rap.call(target, new AddOperation(info));
        switch (result) {
        case 0:
            return;
        case 50:
            throw new RapException(result,
                    "Operation not supported by server.");
        case 2223:
            throw new RapException(result, "Specified group already exists.");
        default:
            throw new RapException(result);
        }
    }

    private static class AddOperation extends Operation {

        private GroupInfo info;

        public AddOperation(GroupInfo info) {
            this.info = info;
            setNumber(NET_GROUP_ADD);
            setParameterDescriptor("WsT");
            setDataDescriptor(info.getDescriptor());
        }

        public void writeRequestParameters(Buffer buffer) {
            buffer.writeShort(info.getLevel());
            buffer.writeShort(0);
        }

        public void writeRequestData(Buffer buffer) {
            info.write(buffer);
        }

    }

}
