/* jcifs smb client library in Java
 * Copyright (C) 2002  "Michael B. Allen" <jcifs at samba dot org>
 *					   "Eric Glass" <jcifs at samba dot org>
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package jcifs.rap;

import java.io.IOException;

import jcifs.smb.NtlmPasswordAuthentication;
import jcifs.smb.Rap;
import jcifs.smb.RapException;
import jcifs.smb.SmbFile;

public class NetServerGetInfo {

    private static final int NET_SERVER_GET_INFO = 13;

    private SmbFile target;

    public NetServerGetInfo(String target, NtlmPasswordAuthentication auth) {
        try {
            this.target = new SmbFile("smb://" + target + "/", auth);
        } catch (Exception ex) {
            throw new IllegalArgumentException("Invalid target: " + ex);
        }
    }

    public ServerInfo getInfo(int level) throws IOException {
        ServerInfo info = (ServerInfo)
                LevelFactory.createInformationLevel(ServerInfo.class, level);
        if (info == null) {
            throw new IllegalArgumentException(
                    "Unsupported information level: " + level);
        }
        InfoOperation operation = new InfoOperation(info);
        int result = Rap.call(target, operation);
        switch (result) {
        case 0:
            return info;
        case 50:
            throw new RapException(result,
                    "Operation not supported by server.");
        default:
            throw new RapException(result);
        }
    }

    private static class InfoOperation extends Operation {

        private ServerInfo info;

        private int availableBytes;

        public InfoOperation(ServerInfo info) {
            this.info = info;
            setNumber(NET_SERVER_GET_INFO);
            setMaxParameterLength(8);
            setParameterDescriptor("WrLh");
            setDataDescriptor(info.getDescriptor());
        }

        public void writeRequestParameters(Buffer buffer) {
            buffer.writeShort(info.getLevel());
            buffer.writeShort(getMaxDataLength());
        }

        public void readResponseParameters(Buffer buffer) {
            availableBytes = buffer.readShort();
        }

        public void readResponseData(Buffer buffer) {
            info.read(buffer);
        }

    }

}
