/* jcifs smb client library in Java
 * Copyright (C) 2002  "Michael B. Allen" <jcifs at samba dot org>
 *					   "Eric Glass" <jcifs at samba dot org>
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package jcifs.rap;

import java.io.IOException;

import jcifs.smb.NtlmPasswordAuthentication;
import jcifs.smb.Rap;
import jcifs.smb.RapException;
import jcifs.smb.SmbFile;

public class NetShareDel {

    private static final int NET_SHARE_DEL = 4;

    private SmbFile target;

    public NetShareDel(String target, NtlmPasswordAuthentication auth) {
        try {
            this.target = new SmbFile("smb://" + target + "/", auth);
        } catch (Exception ex) {
            throw new IllegalArgumentException("Invalid target: " + ex);
        }
    }

    public void deleteShare(String share) throws IOException {
        int result = Rap.call(target, new DeleteOperation(share));
        switch (result) {
        case 0:
            return;
        case 50:
            throw new RapException(result,
                    "Operation not supported by server.");
        default:
            throw new RapException(result);
        }
    }

    private static class DeleteOperation extends Operation {

        private String share;

        public DeleteOperation(String share) {
            this.share = share;
            setNumber(NET_SHARE_DEL);
            setParameterDescriptor("zW");
            setMaxDataLength(200);
        }

        public void writeRequestParameters(Buffer buffer) {
            buffer.writeString(share.length() > 20 ? share.substring(0, 20) :
                    share);
            buffer.writeShort(0);
        }

    }

}
