/* jcifs smb client library in Java
 * Copyright (C) 2002  "Michael B. Allen" <jcifs at samba dot org>
 *					   "Eric Glass" <jcifs at samba dot org>
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package jcifs.rap;

import java.io.IOException;

import jcifs.smb.NtlmPasswordAuthentication;
import jcifs.smb.Rap;
import jcifs.smb.RapException;
import jcifs.smb.SmbFile;

public class NetUserGetGroups {

    private static final int NET_USER_GET_GROUPS = 59;

    private SmbFile target;

    public NetUserGetGroups(String target, NtlmPasswordAuthentication auth) {
        try {
            this.target = new SmbFile("smb://" + target + "/", auth);
        } catch (Exception ex) {
            throw new IllegalArgumentException("Invalid target: " + ex);
        }
    }

    public GroupUsersInfo[] getGroups(String user, int level)
            throws IOException {
        GroupUsersInfo infoTemplate = (GroupUsersInfo)
                LevelFactory.createInformationLevel(GroupUsersInfo.class,
                        level);
        if (infoTemplate == null) {
            throw new IllegalArgumentException(
                    "Unsupported information level.");
        }
        EnumerateOperation operation = new EnumerateOperation(user,
                infoTemplate);
        int result = Rap.call(target, operation);
        switch (result) {
        case 0:
            return operation.groups;
        case 50:
            throw new RapException(result,
                    "Operation not supported by server.");
        default:
            throw new RapException(result);
        }
    }

    private static class EnumerateOperation extends Operation {

        public GroupUsersInfo[] groups;

        private String user;

        private GroupUsersInfo infoTemplate;

        private int entryCount;

        private int availableBytes;

        public EnumerateOperation(String user, GroupUsersInfo infoTemplate) {
            this.user = user;
            this.infoTemplate = infoTemplate;
            setNumber(NET_USER_GET_GROUPS);
            setMaxParameterLength(8);
            setParameterDescriptor("zWrLeh");
            setDataDescriptor(infoTemplate.getDescriptor());
        }

        public void writeRequestParameters(Buffer buffer) {
            buffer.writeString(user.length() > 20 ? user.substring(0, 20) :
                    user);
            buffer.writeShort(infoTemplate.getLevel());
            buffer.writeShort(getMaxDataLength());
        }

        public void readResponseParameters(Buffer buffer) {
            entryCount = buffer.readShort();
            availableBytes = buffer.readShort();
        }

        public void readResponseData(Buffer buffer) {
            groups = new GroupUsersInfo[entryCount];
            for (int i = 0; i < entryCount; i++) {
                groups[i] = (GroupUsersInfo) infoTemplate.clone();
                groups[i].read(buffer);
            }
        }

    }

}
