/* jcifs smb client library in Java
 * Copyright (C) 2002  "Michael B. Allen" <jcifs at samba dot org>
 *					   "Eric Glass" <jcifs at samba dot org>
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package jcifs.rap;

import java.util.Hashtable;
import java.util.Map;

import jcifs.smb.NtlmPasswordAuthentication;

/**
 * Provides simplified interfaces for managing servers.
 */
public class ServerUtilities {

    private static final Map PDC_MAP = new Hashtable();

    public static String getPrimaryController(String domain,
            NtlmPasswordAuthentication auth) {
        try {
            String pdc = (String) PDC_MAP.get(domain);
            if (pdc != null) return pdc;
            ServerInfo[] controllers = new NetServerEnum(domain,
                    auth).getServers(0, ServerInfo.SV_TYPE_DOMAIN_CTRL, domain);
            pdc = (controllers == null || controllers.length == 0) ? null :
                    controllers[0].name;
            if (pdc != null) PDC_MAP.put(domain, pdc);
            return pdc;
        } catch (Exception ex) {
            return null;
        }
    }

    public String[] getServers(String domain, int serverType,
            NtlmPasswordAuthentication auth) {
        try {
            ServerInfo[] servers = new NetServerEnum(domain,
                    auth).getServers(0, serverType, domain);
            if (servers == null) return null;
            int length = servers.length;
            String[] names = new String[length];
            for (int i = 0; i < length; i++) names[i] = servers[i].name;
            return names;
        } catch (Exception ex) {
            return null;
        }
    }

}
