/* jcifs smb client library in Java
 * Copyright (C) 2002  "Michael B. Allen" <jcifs at samba dot org>
 *					   "Eric Glass" <jcifs at samba dot org>
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package jcifs.rap;

import jcifs.smb.NtlmPasswordAuthentication;

/**
 * Provides simplified interfaces for managing shares on a server.
 */
public class ShareUtilities {

    public static String[] getShares(String server,
            NtlmPasswordAuthentication auth) {
        try {
            ShareInfo[] shares = new NetShareEnum(server, auth).getShares(0);
            if (shares == null) return new String[0];
            int length = shares.length;
            String[] names = new String[length];
            for (int i = 0; i < length; i++) names[i] = shares[i].name;
            return names;
        } catch (Exception ex) {
            return null;
        }
    }

    public static boolean create(String server, String share, String path,
            NtlmPasswordAuthentication auth) {
        try {
            ShareInfo2 info = new ShareInfo2();
            info.name = share;
            info.path = path;
            new NetShareAdd(server, auth).createShare(info);
            return true;
        } catch (Exception ex) {
            return false;
        }
    }

    public static boolean delete(String server, String share,
            NtlmPasswordAuthentication auth) {
        try {
            new NetShareDel(server, auth).deleteShare(share);
            return true;
        } catch (Exception ex) {
            return false;
        }
    }

}
