/* jcifs smb client library in Java
 * Copyright (C) 2002  "Michael B. Allen" <jcifs at samba dot org>
 *					   "Eric Glass" <jcifs at samba dot org>
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package jcifs.rap;

import jcifs.smb.NtlmPasswordAuthentication;

/**
 * Provides simplified interfaces for managing domain and machine-local
 * users.
 */
public class UserUtilities {

    // password changing requires an available JCE ARC4 cipher implementation.
    private static boolean passwordChangeSupported;

    static {
        try {
            Class cipher = Class.forName("javax.crypto.Cipher");
            cipher.getMethod("getInstance",
                    new Class[] { String.class }).invoke(null,
                            new Object[] { "ARC4" });
            passwordChangeSupported = true;
        } catch (Exception ex) {
            passwordChangeSupported = false;
        }
    }

    public static String[] getUsers(String target,
            NtlmPasswordAuthentication auth) {
        try {
            String server = ServerUtilities.getPrimaryController(target, auth);
            if (server == null) server = target;
            UserInfo[] users = new NetUserEnum(server, auth).getUsers(0);
            if (users == null) return new String[0];
            int length = users.length;
            String[] names = new String[length];
            for (int i = 0; i < length; i++) names[i] = users[i].name;
            return names;
        } catch (Exception ex) {
            return null;
        }
    }

    public static boolean changePassword(NtlmPasswordAuthentication account,
            String password, NtlmPasswordAuthentication auth) {
        try {
            if (!passwordChangeSupported) return false;
            String target = account.getDomain();
            String server = ServerUtilities.getPrimaryController(target, auth);
            if (server == null) server = target;
            new SamOemChangePassword(server, auth).changePassword(
                    account.getUsername(), account.getPassword(), password);
            return true;
        } catch (Exception ex) {
            return false;
        }
    }

    public static NtlmPasswordAuthentication create(String target,
            String user, String password, NtlmPasswordAuthentication auth) {
        try {
            String server = ServerUtilities.getPrimaryController(target, auth);
            if (server == null) server = target;
            UserInfo1 info = new UserInfo1();
            info.name = user;
            info.privilegeLevel = 1;
            info.password = "";
            new NetUserAdd(server, auth).createUser(info);
            NtlmPasswordAuthentication account = new NtlmPasswordAuthentication(
                    target, user, "");
            // NetUserAdd is kind of funky; doesn't appear to work with
            // a plaintext password specified, so create with an empty
            // password and then change it.
            if (password == null) password = "";
            if (passwordChangeSupported && !"".equals(password)) {
                try {
                    new SamOemChangePassword(server, auth).changePassword(
                            user, "", password);
                    account = new NtlmPasswordAuthentication(target, user,
                            password);
                } catch (Exception ignore) { ignore.printStackTrace(); }
            }
            return account;
        } catch (Exception ex) {
            return null;
        }
    }

    public static boolean delete(String target, String user,
            NtlmPasswordAuthentication auth) {
        try {
            String server = ServerUtilities.getPrimaryController(target, auth);
            if (server == null) server = target;
            new NetUserDel(server, auth).deleteUser(user);
            return true;
        } catch (Exception ex) {
            return false;
        }
    }

    public static String[] getGroups(String target, String user,
            NtlmPasswordAuthentication auth) {
        try {
            String server = ServerUtilities.getPrimaryController(target, auth);
            if (server == null) server = target;
            GroupUsersInfo[] groups = new NetUserGetGroups(server,
                    auth).getGroups(user, 0);
            if (groups == null) return new String[0];
            int length = groups.length;
            String[] names = new String[length];
            for (int i = 0; i < length; i++) names[i] = groups[i].name;
            return names;
        } catch (Exception ex) {
            return null;
        }
    }

}
